#include <cstdio>
using namespace std;

#include "opencv2/highgui/highgui.hpp"
#include "opencv2/imgproc/imgproc.hpp"
using namespace cv;

class LoopedVideo : public VideoCapture {
public:
	LoopedVideo() {}
	virtual bool grab() {
		if(VideoCapture::grab()==false) {
			rewind();
		}
		return true;
	}
	void rewind() {
		set(CV_CAP_PROP_POS_FRAMES,0);
	}
};

const int FPS = 30;
VideoCapture camera;
Mat inputImage;
Mat savedBackgroundImage;
Mat differenceImage;
Mat grayscaleDifferenceImage;
Mat thresholdedDifferenceImage;
int foregroundThreshold;
Mat fixedDifferenceImage;
Mat foregroundMask;
Mat foregroundImage;
Mat currentBackgroundImage;
Mat backgroundImage;
const string staticBackgroundImagePath = "forest.jpg";
Mat resultImage;

LoopedVideo backgroundMovie;
const string dynamicBackgroundMoviePath = "fishie.avi";

const string recordedMoviePath = "recorded.avi";
VideoWriter recordedMovie;
bool isRecording;

void showUsage() {
	printf("Press 'b' to set background\n");
	printf("Press 'k' to terminate\n");
	printf("Press 'a' to increase foregroundThreshold\n");
	printf("Press 'z' to decrease foregroundThreshold\n");
	printf("Press 'r' to start and stop recording\n");
}
 
void processKey(char key) {
	switch(key) {
		case 'b': 
			inputImage.copyTo(savedBackgroundImage);
			imshow("Background",savedBackgroundImage);
			break;
		case 'a':
			foregroundThreshold+=1;
			printf("foregroundThreshold = %d\n", foregroundThreshold);
			break;
		case 'z':
			if(foregroundThreshold>1) foregroundThreshold -=1;
			printf("foregroundThreshold = %d\n", foregroundThreshold);
			break;
		case 'r':
			if(isRecording == true) {
				isRecording = false;
				printf("Recording has been stoped!\n");
			}
			else {
				recordedMovie.open(recordedMoviePath,0,FPS,Size(640,480));
				isRecording = true;
				printf("Recording has started!\n");
			}
			break;
		}
}

int main(int argc, char *argv[]) {
	camera.open(0);
	backgroundMovie.open(dynamicBackgroundMoviePath);

	if(!camera.isOpened()){
		printf("Cannot connect to the camera!\n"); 
		getchar();
	}
	else {
		printf("Connection established!\n");
		showUsage();
		camera.set(CV_CAP_PROP_FRAME_WIDTH,640);
		camera.set(CV_CAP_PROP_FRAME_HEIGHT,480);
		char key=0;
		while(key!='k') {
			camera >> inputImage;
			backgroundMovie >> currentBackgroundImage;
			imshow("Camera Image", inputImage);
			if(savedBackgroundImage.data!=NULL) {
				absdiff(inputImage, savedBackgroundImage,differenceImage);
				imshow("Difference Image",differenceImage);
				cvtColor(differenceImage,grayscaleDifferenceImage,CV_BGR2GRAY);
				threshold(grayscaleDifferenceImage,thresholdedDifferenceImage, foregroundThreshold,255,THRESH_BINARY);
				imshow("Foreground Mask", thresholdedDifferenceImage);
				morphologyEx(thresholdedDifferenceImage,fixedDifferenceImage,MORPH_OPEN,Mat(),Point(-1,-1),1);
				morphologyEx(fixedDifferenceImage,fixedDifferenceImage,MORPH_CLOSE,Mat(),Point(-1,-1),2);
				imshow("Fixed Foreground Mask", fixedDifferenceImage);
				cvtColor(fixedDifferenceImage,foregroundMask,CV_GRAY2BGR);
				foregroundImage = foregroundMask & inputImage;
				imshow("Foreground Image",foregroundImage);

				if(currentBackgroundImage.data!=NULL) {
					backgroundImage = currentBackgroundImage - foregroundMask;
					resultImage = foregroundImage | backgroundImage;
					medianBlur(resultImage,resultImage,3);
					imshow("Result!",resultImage);

					if(isRecording==true) {
						recordedMovie << resultImage;
					}
				}
			}
			key = waitKey(1000/FPS);
			processKey(key);
		}
	}
	return 0;
}